@@ -1,13 +1,17 @@
+import logging
+import socket
+import time
+
 from satella.coding import silence_excs
 from satella.coding.concurrent import TerminableThread
-from ..exceptions import InvalidFrame
-import socket
-import logging
 
-from modbus_tcp_server.datagrams import MODBUSTCPMessage
+from ..datagrams import MODBUSTCPMessage
+from ..exceptions import InvalidFrame
 
 logger = logging.getLogger(__name__)
 
+MAX_TIME_WITHOUT_ACTIVITY = 60.0    # 60 seconds
+
 
 class ConnectionThread(TerminableThread):
     def __init__(self, sock, addr, server):
@@ -18,18 +22,22 @@ class ConnectionThread(TerminableThread):
         self.socket.setblocking(True)
         self.socket.settimeout(5)
         self.buffer = bytearray()
+        self.last_activity = time.monotonic()
 
     def cleanup(self):
         self.socket.close()
 
     @silence_excs(socket.timeout)
     def loop(self):
+        # Check for termination conditions
         # noinspection PyProtectedMember
         if self.server._terminating:
-            self.terminate()
-            return
-        data = self.socket.recv(128)
+            raise socket.error()
+        if time.monotonic() - self.last_activity > MAX_TIME_WITHOUT_ACTIVITY:
+            raise socket.error()
 
+        # Get the data
+        data = self.socket.recv(128)
         if not data:
             raise socket.error()
         logger.info('Received %s', repr(data))
@@ -39,11 +47,15 @@ class ConnectionThread(TerminableThread):
         while self.buffer:
             try:
                 packet = MODBUSTCPMessage.from_bytes(self.buffer)
-                del self.buffer[:len(packet)]
             except ValueError:
                 break
-
+            del self.buffer[:len(packet)]
             msg = self.server.process_message(packet)
             b = bytes(msg)
             logger.debug('Sent %s', repr(b))
-            self.socket.sendall(b)
+            try:
+                self.socket.sendall(b)
+            except socket.timeout:
+                # Timeout on sendall means the client is extremely slow
+                raise socket.error()
+            self.last_activity = time.monotonic()
